<?php
use WP_Error;
use WP_REST_Request;

class PicsRestAPI
{
	public const BASE = 'picsio';
	public const VERSION = 'v1';

	public function register(): void
	{
		add_action('rest_api_init', [$this, 'register_gallery_routes']);
	}


	/**
	 * Register gallery routes
	 * @return void
	 */
	public function register_gallery_routes(): void
	{
		register_rest_route(self::BASE . '/' . self::VERSION, '/gallery/upload_assets', [
			'methods'             => 'POST',
			'callback'            => [$this, 'upload_assets'],
			'permission_callback' => [$this, 'upload_assets_permission'],
			'args'                => array(
				'downloadLinks' => array(
					'required' => true,
					'validate_callback' => function ($urls, $request, $key) {
						if (empty($urls)) {
							return new WP_Error(
								'rest_forbidden',
								__('The value of downloadLinks field is empty.', 'pics')
							);
						}

						if (!is_array($urls)) {
							return new WP_Error(
								'rest_forbidden',
								__('The value of downloadLinks field is invalid.', 'pics')
							);
						}

						foreach ($urls as $url) {
							if (filter_var($url, FILTER_VALIDATE_URL) === false) {
								return new WP_Error(
									'rest_forbidden',
									__('One of the download links is invalid.', 'pics')
								);
							}

							if (!$this->file_exists_by_url($url)) {
								return new WP_Error(
									'rest_forbidden',
									__('One of the download links is invalid.', 'pics')
								);
							}
						}

						return true;
					}
				),
			),
		]);
	}

	/* -------------------------------------------------------------- //
	* ----------------------- GALLERY ROUTES ------------------------ //
	* --------------------------------------------------------------- */

	public function upload_assets(WP_REST_Request $request)
	{
		$downloadLinks = $request->get_param('downloadLinks');

		$uploadDir = wp_upload_dir();

		if (!is_writable($uploadDir['path'])) {
			return rest_ensure_response([
				'success' => false,
				'data'    => [
					'message' => __('The upload directory is not writtable.', 'pics')
				]
			]);
		}

		include_once(ABSPATH . 'wp-admin/includes/image.php');
		$files = array();

		foreach ($downloadLinks as $asset_url) {
			if (stripos($asset_url, 'googleapis') !== false) {
				$filePath = $this->download_file_from_google_drive_and_save($asset_url, $uploadDir['path']);
			} elseif (stripos($asset_url, 'amazonaws') !== false) {
				$filePath = $this->download_file_from_amazon_aws_and_save($asset_url, $uploadDir['path']);
			} else {
				$filePath = $this->download_file_and_save($asset_url, $uploadDir['path']);
			}

			$filename = basename($filePath);
			$wp_filetype = wp_check_filetype($filename);
			if (empty($wp_filetype['type'])) {
				$ext = pathinfo($filename, PATHINFO_EXTENSION);

				return new WP_Error('forbidden_image_mime_type', sprintf(__("The image format '%s' is not allowed by WordPress!", 'pics'), $ext), array('status' => 400));
			}

			$attachment = array(
				'post_mime_type' => $wp_filetype['type'],
				'post_title'     => $filename,
				'post_content'   => '',
				'post_status'    => 'inherit'
			);

			// create media file
			$attach_id = wp_insert_attachment($attachment, $filePath);

			// generate metadata for a file
			$imagenew     = get_post($attach_id);
			$fullsizepath = get_attached_file($imagenew->ID);
			$attach_data  = wp_generate_attachment_metadata($attach_id, $fullsizepath);
			wp_update_attachment_metadata($attach_id, $attach_data);
			array_push($files, $attach_data);
		}

		return rest_ensure_response([
			'success' => true,
			'files'   => $files
		]);
	}

	/* -------------------------------------------------------------- //
	* ------------------- PERMISSION CHECKS ------------------------- //
	* --------------------------------------------------------------- */

	/**
	 * Check if user is logged in
	 *
	 * @param WP_REST_Request $request HTTP Request.
	 *
	 * @return bool
	 */
	public function is_user_logged_in(WP_REST_Request $request): bool
	{
		if (!is_user_logged_in()) {
			return false;
		}

		if (!wp_verify_nonce($request->get_header('X-WP-NONCE'), 'wp_rest')) {
			return false;
		}

		return true;
	}

	public function upload_assets_permission(WP_REST_Request $request): bool
	{
		return $this->is_user_logged_in($request);
	}

	/* -------------------------------------------------------------- //
	* --------------------------- HELPERS --------------------------- //
	* --------------------------------------------------------------- */

	private function file_exists_by_url(string $url): string
	{
		if (stripos($url, 'googleapis') !== false) {
			parse_str(parse_url($url)['query'], $query);
			if (empty($query['t'])) {
				return false;
			}

			/** @var array|WP_Error $response */
			$response = wp_remote_head($url, [
				'headers' => array(
					'Authorization' => 'Bearer ' . $query['t']
				)
			]);

			if (is_wp_error($response)) {
				return new WP_Error('fetch_failed', $response->get_error_message());
			}

			return wp_remote_retrieve_response_code($response) === 200;
		}

		$headers = get_headers($url);

		return stripos($headers[0], "200 OK") !== false ? true : false;
	}

	private function download_file_and_save(string $url, string $path)
	{
		$imagetype = end(explode('/', getimagesize($url)['mime']));
		$uniq_name = date('dmY') . '' . (int) microtime(true);
		$filename  = $uniq_name . '.' . $imagetype;

		$uploadFilePath = $path . '/' . $filename;

		// download file
		$data = wp_remote_retrieve_body(wp_remote_get($url));

		// save it into upload directory
		$savefile = fopen($uploadFilePath, 'w');
		fwrite($savefile, $data);
		fclose($savefile);

		return $uploadFilePath;
	}

	private function download_file_from_amazon_aws_and_save(string $url, string $path)
	{
		return $this->download_file_and_save($url, $path);
	}

	private function download_file_from_google_drive_and_save(string $url, string $path)
	{
		parse_str(parse_url($url)['query'], $query);
		if (empty($query['t'])) {
			return false;
		}

		/** @var array|WP_Error $response */
		$response = wp_remote_get($url, [
			'headers' => array(
				'Authorization' => 'Bearer ' . $query['t']
			)
		]);
		if (is_wp_error($response)) {
			return new WP_Error('fetch_failed', $response->get_error_message());
		}

		$mimeType = wp_remote_retrieve_header($response, 'content-type');
		error_log($mimeType);
		$ext = $this->convert_image_mime_type_to_file_extenstion($mimeType);
		$uniq_name = date('dmY') . '' . (int) microtime(true);
		$filename  = $uniq_name . '.' . $ext;

		$data = wp_remote_retrieve_body($response);

		$uploadFilePath = $path . '/' . $filename;

		$savefile = fopen($uploadFilePath, 'w');
		fwrite($savefile, $data);
		fclose($savefile);

		return $uploadFilePath;
	}

	private function convert_image_mime_type_to_file_extenstion(string $mimeType)
	{
		$imageMimeTypes = pics_get_image_mime_types();

		return !empty($imageMimeTypes[$mimeType]) ? $imageMimeTypes[$mimeType] : '';
	}
}

$picsRestApi = new PicsRestAPI();
$picsRestApi->register();
